//==================================================================================
// originally from
// 	SDL/SGE (sge_FloodFill: Fast non-recursive flood fill)
//
// Algorithm originally written by
//  Paul Heckbert, 13 Sept 1982, 28 Jan 1987
//==================================================================================
/** \file cam_object.c
 *  \brief CAM-Functions to find Objects in a Heightmap
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <glade/glade.h>

#include "cam.h"
#include "cam_object.h"

typedef struct _seg seg;
struct _seg {
	int y, xl, xr, dy;
};

#define STACK_MAX 10000
#define PUSH(Y, XL, XR, DY){\
	if (sp<stack+STACK_MAX && Y+(DY)>=0 && Y+(DY)<HeightMap->height){\
		sp->y = Y;\
		sp->xl = XL;\
		sp->xr = XR;\
		sp->dy = DY;\
		sp++;\
	}\
}
#define POP(Y, XL, XR, DY){\
	sp--;\
	DY = sp->dy;\
	Y = sp->y + sp->dy;\
	XL = sp->xl;\
	XR = sp->xr;\
}

//! returns a new HeightMap with the founded Object on XY in the given HeightMap
CAM__HeightMap *CAM__HeightMap_MoveObject(CAM__HeightMap *HeightMap, gint x, gint y) {
	CAM__HeightMap *HeightMap_New;
	gint cx;
	gint cy;
	gboolean found = FALSE;
	if (x < 0 || x >= HeightMap->width || y < 0 || y >= HeightMap->height) {
		return NULL;
	}
	if ((HeightMap_New = CAM__HeightMap_create(HeightMap->width, HeightMap->height)) == NULL) {
		return NULL;
	}
	for (cy = 0; cy < HeightMap->height; cy += 1) {
		for (cx = 0; cx < HeightMap->width; cx += 1) {
			HeightMap_New->data[(cy * HeightMap_New->width) + cx] = 65535;
		}
	}

	gint l, x1, x2, dy;
	seg stack[STACK_MAX], *sp = stack;	/* stack of filled segments */
	PUSH(y, x, x, 1);			/* needed in some cases */
	PUSH(y + 1, x, x, -1);		/* seed segment (popped 1st) */
	while (sp > stack) {
		POP(y, x1, x2, dy);
		for (x = x1; x >= 0; x--) {
			if (HeightMap->data[(y * HeightMap->width) + x] == 65535) {
				break;
			}
			HeightMap_New->data[(y * HeightMap->width) + x] = HeightMap->data[(y * HeightMap->width) + x];
			HeightMap->data[(y * HeightMap->width) + x] = 65535;
			//fprintf(stderr, "## %i %i %i \n", x, y, HeightMap->data[(y * HeightMap->width) + x]);
			found = TRUE;
		}
		if (x >= x1) {
			goto skip;
		}
		l = x + 1;
		if (l < x1) {
			PUSH(y, l, x1 - 1, -dy);		/* leak on left? */
		}
		x = x1 + 1;
		do {
			for (; x < HeightMap->width; x++) {
				if (HeightMap->data[(y * HeightMap->width) + x] == 65535) {
					break;
				}
				HeightMap_New->data[(y * HeightMap->width) + x] = HeightMap->data[(y * HeightMap->width) + x];
				HeightMap->data[(y * HeightMap->width) + x] = 65535;
				found = TRUE;
			}
			PUSH(y, l, x - 1, dy);
			if (x > x2 + 1) {
				PUSH(y, x2 + 1, x - 1, -dy);	/* leak on right? */
			}
skip:		
			for (x++; x <= x2; x++) {
				if (CAM__HeightMap_CheckPixel(HeightMap, x, y) != 65535) {
					break;
				}
			}
			l = x;
		} while (x <= x2);
	}
	if (found == FALSE) {
		CAM__Heightmap_delete(HeightMap_New);
		return NULL;
	}
	return HeightMap_New;
}

//! returns a new HeightMap with the founded Object on XY in the given HeightMap (Inverted-Version)
CAM__HeightMap *CAM__HeightMap_MoveObject_Invert(CAM__HeightMap *HeightMap, gint x, gint y) {
	CAM__HeightMap *HeightMap_New;
	gint cx;
	gint cy;
	gboolean found = FALSE;
	if (x < 0 || x >= HeightMap->width || y < 0 || y >= HeightMap->height) {
		return NULL;
	}
	if ((HeightMap_New = CAM__HeightMap_create(HeightMap->width, HeightMap->height)) == NULL) {
		return NULL;
	}
	for (cy = 0; cy < HeightMap->height; cy += 1) {
		for (cx = 0; cx < HeightMap->width; cx += 1) {
			HeightMap_New->data[(cy * HeightMap_New->width) + cx] = 0;
		}
	}

	gint l, x1, x2, dy;
	seg stack[STACK_MAX], *sp = stack;	/* stack of filled segments */
	PUSH(y, x, x, 1);			/* needed in some cases */
	PUSH(y + 1, x, x, -1);		/* seed segment (popped 1st) */
	while (sp > stack) {
		POP(y, x1, x2, dy);
		for (x = x1; x >= 0; x--) {
			if (HeightMap->data[(y * HeightMap->width) + x] == 0) {
				break;
			}
			HeightMap_New->data[(y * HeightMap->width) + x] = 65535;
			HeightMap->data[(y * HeightMap->width) + x] = 0;
			//fprintf(stderr, "## %i %i %i \n", x, y, HeightMap->data[(y * HeightMap->width) + x]);
			found = TRUE;
		}
		if (x >= x1) {
			goto skip;
		}
		l = x + 1;
		if (l < x1) {
			PUSH(y, l, x1 - 1, -dy);		/* leak on left? */
		}
		x = x1 + 1;
		do {
			for (; x < HeightMap->width; x++) {
				if (HeightMap->data[(y * HeightMap->width) + x] == 0) {
					break;
				}
				HeightMap_New->data[(y * HeightMap->width) + x] = 65535;
				HeightMap->data[(y * HeightMap->width) + x] = 0;
				found = TRUE;
			}
			PUSH(y, l, x - 1, dy);
			if (x > x2 + 1) {
				PUSH(y, x2 + 1, x - 1, -dy);	/* leak on right? */
			}
skip:		
			for (x++; x <= x2; x++) {
				if (CAM__HeightMap_CheckPixel(HeightMap, x, y) != 0) {
					break;
				}
			}
			l = x;
		} while (x <= x2);
	}
	if (found == FALSE) {
		CAM__Heightmap_delete(HeightMap_New);
		return NULL;
	}
	return HeightMap_New;
}

CAM__HeightMap *CAM__HeightMap_FloodFill(CAM__HeightMap *HeightMap, gint x, gint y, gint color) {
	CAM__HeightMap *HeightMap_New;
	gint cx;
	gint cy;
	HeightMap_New = malloc(sizeof(CAM__HeightMap));
	HeightMap_New->width = HeightMap->width;
	HeightMap_New->height = HeightMap->height;
	HeightMap_New->data = malloc(HeightMap_New->width * HeightMap_New->height * sizeof(gint));
	for (cy = 0; cy < HeightMap->height; cy += 1) {
		for (cx = 0; cx < HeightMap->width; cx += 1) {
//			HeightMap_New->data[(cy * HeightMap_New->width) + cx] = HeightMap->data[(cy * HeightMap->width) + cx];
			HeightMap_New->data[(cy * HeightMap_New->width) + cx] = 0;
		}
	}

	gint l, x1, x2, dy;
	gint oc;						/* old pixel color */
	seg stack[STACK_MAX], *sp = stack;	/* stack of filled segments */
	if (x < 0 || x > HeightMap->width || y < 0 || y > HeightMap->height) {
		return HeightMap_New;
	}
	oc = CAM__HeightMap_CheckPixel(HeightMap, x, y);	/* read color at seed point */
	if (oc == color) {
		return HeightMap_New;
	}
	PUSH(y, x, x, 1);			/* needed in some cases */
	PUSH(y + 1, x, x, -1);		/* seed segment (popped 1st) */
	while (sp > stack) {
		POP(y, x1, x2, dy);
		for (x = x1; x >= 0; x--) {
			if (CAM__HeightMap_CheckPixel(HeightMap, x, y) != oc) {
				break;
			}
			HeightMap->data[(y * HeightMap->width) + x] = color;
			HeightMap_New->data[(y * HeightMap->width) + x] = color;
		}
		if (x >= x1) {
			goto skip;
		}
		l = x + 1;
		if (l < x1) {
			PUSH(y, l, x1-1, -dy);		/* leak on left? */
		}
		x = x1+1;
		do {
			for (; x<HeightMap->width; x++) {
				if ( CAM__HeightMap_CheckPixel(HeightMap, x, y) != oc ) {
					break;
				}
				HeightMap->data[(y * HeightMap->width) + x] = color;
				HeightMap_New->data[(y * HeightMap->width) + x] = color;
			}
			PUSH(y, l, x - 1, dy);
			if (x > x2 + 1) {
				PUSH(y, x2 + 1, x - 1, -dy);	/* leak on right? */
			}
skip:		
			for (x++; x <= x2; x++) {
				if (CAM__HeightMap_CheckPixel(HeightMap, x, y) == oc) {
					break;
				}
			}
			l = x;
		} while (x<=x2);
	}
	return HeightMap_New;
}
