/** \file cam_setup.c
 *  \brief GUI & Functions for the Cam-Setup
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <glade/glade.h>

#include "cam_setup.h"
#include "cnc_output.h"

gint cnc_z_steps;
gint cnc_active;
gint cnc_last_tool_offset;
gint cnc_last_updown;
gint cnc_last_x;
gint cnc_last_y;
gint cnc_last_z;
gint cnc_do_cut;
gint cnc_do_fill;
CamSetup *camsetup__camset;

//! create a new Cam-Setup-Context
CamSetup *camsetup__init (void) {
	camsetup__camset = malloc(sizeof(CamSetup));
	camsetup__camset->xml = glade_xml_new(PACKAGE_SOURCE_DIR"/camsetup.glade", NULL, NULL);
	glade_xml_signal_autoconnect(camsetup__camset->xml);
	camsetup__camset->window = glade_xml_get_widget(camsetup__camset->xml, "camsetupdialog");
	gtk_widget_hide(camsetup__camset->window);

	camsetup__values_default();
	camsetup__values_set();
	
	return camsetup__camset;
}


//! strip space-chars from the begin and the and of a string
void camsetup__trim_str(gchar *old_string, gchar *new_string) {
	gint n1 = 0;
	gint n2 = 0;
	while (old_string[n1] == ' ' || old_string[n1] == '\t' || old_string[n1] == '\n') {
		n1++;
	}
	while (old_string[n1] != 0) {
		new_string[n2++] = old_string[n1++];
	}
	new_string[n2--] = 0;
	while (new_string[n2] == ' ' || new_string[n2] == '\t' || new_string[n2] == '\n') {
		new_string[n2] = 0;
		n2--;
	}
}

//! load the camsetup rc-file
void camsetup__load (gchar *filename) {
	FILE *file;
	gchar *value_ptr;
	gchar line_buffer[1024];
	gchar value[1024];
	gchar name[1024];
	if ((file = fopen(filename, "r")) != NULL) {
		while (!feof(file) && fgets(line_buffer, 1023, file) != 0) {
			if ((value_ptr = strstr(line_buffer, "=")) != 0) {
				line_buffer[value_ptr - line_buffer] = 0;
				camsetup__trim_str(line_buffer, name);
				camsetup__trim_str(value_ptr + 1, value);
				if (name[0] == 0) {
				} else if (strcmp(name, "zoom") == 0) {
					camsetup__camset->zoom = atof(value);
				} else if (strcmp(name, "z_start") == 0) {
					camsetup__camset->z_start = atoi(value);
				} else if (strcmp(name, "z_stop") == 0) {
					camsetup__camset->z_stop = atoi(value);
				} else if (strcmp(name, "z_over") == 0) {
					camsetup__camset->z_over = atoi(value);
				} else if (strcmp(name, "z_steps") == 0) {
					camsetup__camset->z_steps = atoi(value);
				} else if (strcmp(name, "z_step") == 0) {
					camsetup__camset->z_step = atoi(value);
				} else if (strcmp(name, "output") == 0) {
					camsetup__camset->output = atoi(value);
				} else if (strcmp(name, "tool") == 0) {
					camsetup__camset->tool = atoi(value);
				} else if (strcmp(name, "axis_x0_speed") == 0) {
					camsetup__camset->axis_x[0].speed = atoi(value);
				} else if (strcmp(name, "axis_y0_speed") == 0) {
					camsetup__camset->axis_y[0].speed = atoi(value);
				} else if (strcmp(name, "axis_z0_speed") == 0) {
					camsetup__camset->axis_z[0].speed = atoi(value);
				} else if (strcmp(name, "axis_z0_tool.speed") == 0) {
					camsetup__camset->axis_z[0].tool.speed = atoi(value);
				} else if (strcmp(name, "axis_z0_tool.head.radius") == 0) {
					camsetup__camset->axis_z[0].tool.head.radius = atoi(value);
				} else if (strcmp(name, "axis_z0_tool.head.type") == 0) {
					camsetup__camset->axis_z[0].tool.head.type = atoi(value);
				} else if (strcmp(name, "axis_z1_speed") == 0) {
					camsetup__camset->axis_z[1].speed = atoi(value);
				} else if (strcmp(name, "axis_z1_tool.speed") == 0) {
					camsetup__camset->axis_z[1].tool.speed = atoi(value);
				} else if (strcmp(name, "axis_z1_tool.head.radius") == 0) {
					camsetup__camset->axis_z[1].tool.head.radius = atoi(value);
				} else if (strcmp(name, "axis_z1_tool.head.type") == 0) {
					camsetup__camset->axis_z[1].tool.head.type = atoi(value);
				}
			}
		}
		fclose(file);
	} else {
		fprintf(stderr, "Can not load '%s'\n", filename);
	}
}

//! save the camsetup rc-file
void camsetup__save (gchar *filename) {
	FILE *file;
	if ((file = fopen(filename, "w")) != NULL) {
		fprintf(file, "## CAM-Setup ##\n");
		fprintf(file, "## written by cnc2d/camsetup ##\n");
		fprintf(file, "\n");
		fprintf(file, "zoom=%0.2f\n", camsetup__camset->zoom);
		fprintf(file, "z_start=%i\n", camsetup__camset->z_start);
		fprintf(file, "z_stop=%i\n", camsetup__camset->z_stop);
		fprintf(file, "z_over=%i\n", camsetup__camset->z_over);
		fprintf(file, "z_steps=%i\n", camsetup__camset->z_steps);
		fprintf(file, "z_step=%i\n", camsetup__camset->z_step);
		fprintf(file, "output=%i\n", camsetup__camset->output);
		fprintf(file, "tool=%i\n", camsetup__camset->tool);
		fprintf(file, "axis_x0_speed=%i\n", camsetup__camset->axis_x[0].speed);
		fprintf(file, "axis_y0_speed=%i\n", camsetup__camset->axis_y[0].speed);
		fprintf(file, "axis_z0_speed=%i\n", camsetup__camset->axis_z[0].speed);
		fprintf(file, "axis_z0_tool.speed=%i\n", camsetup__camset->axis_z[0].tool.speed);
		fprintf(file, "axis_z0_tool.head.radius=%i\n", camsetup__camset->axis_z[0].tool.head.radius);
		fprintf(file, "axis_z0_tool.head.type=%i\n", camsetup__camset->axis_z[0].tool.head.type);
		fprintf(file, "axis_z1_speed=%i\n", camsetup__camset->axis_z[1].speed);
		fprintf(file, "axis_z1_tool.speed=%i\n", camsetup__camset->axis_z[1].tool.speed);
		fprintf(file, "axis_z1_tool.head.radius=%i\n", camsetup__camset->axis_z[1].tool.head.radius);
		fprintf(file, "axis_z1_tool.head.type=%i\n", camsetup__camset->axis_z[1].tool.head.type);
		fprintf(file, "\n");
		fclose(file);
	} else {
		fprintf(stderr, "Can not save '%s'\n", filename);
	}
}

//! shows the Cam-Setup-Dialog
void camsetup__show (void) {
	camsetup__values_set();
	gtk_window_set_title(GTK_WINDOW(camsetup__camset->window), "CAM-Setup");
	gtk_widget_show(camsetup__camset->window);
}

//! hides the Cam-Setup-Dialog
void camsetup__hide (void) {
	gtk_widget_hide(camsetup__camset->window);
}

//! sets the default Cam-Setup-Values
void camsetup__values_default (void) {
	camsetup__camset->zoom = 10.0;    // Ausgabe-Skalierung
	camsetup__camset->z_start = 2000; // 20.00mm über 0 (Oberkante des Werkstücks)
	camsetup__camset->z_stop = 0;     // 2.00mm über 0 (Unterkante des Werkstücks)
	camsetup__camset->z_over = 300;   // 3.00mm über Start (Schnelllauf-Possition)
	camsetup__camset->z_steps = 400;  // 1.00mm (Maximale Schnitttiefe)
	camsetup__camset->z_step = 0;     // FOR INTERNAL USE (Aktuelle Z-Possition)
	camsetup__camset->output = 5;     // Ausgabe-Treiber

	camsetup__camset->tool = 0;
	camsetup__camset->axis_x[0].speed = 100;
	camsetup__camset->axis_y[0].speed = 100;

	camsetup__camset->axis_z[0].speed = 100;
	camsetup__camset->axis_z[0].tool.speed = 100;
	camsetup__camset->axis_z[0].tool.head.radius = 168;
	camsetup__camset->axis_z[0].tool.head.type = 0;

	camsetup__camset->axis_z[1].speed = 100;
	camsetup__camset->axis_z[1].tool.speed = 100;
	camsetup__camset->axis_z[1].tool.head.radius = 110;
	camsetup__camset->axis_z[1].tool.head.type = 0;

	CNC__SetZoom(camsetup__camset->zoom);
	CNC__SetZoom_Z(1.0);
	CNC__SetOutput(camsetup__camset->output);

	gchar rcfile[1024];
	sprintf(rcfile, "%s/.camrc", g_get_home_dir());
	camsetup__load(rcfile);

}

//! sets the Cam-Setup-Values in the Dialog-Gui
void camsetup__values_set (void) {
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_zoom")), (gdouble)camsetup__camset->zoom);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_max_z")), ((gdouble)camsetup__camset->z_start));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_min_z")), ((gdouble)camsetup__camset->z_stop));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_over_z")), ((gdouble)camsetup__camset->z_over));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_steps_z")), ((gdouble)camsetup__camset->z_steps));
	gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_output")), camsetup__camset->output);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_axisx_speed")), (gdouble)camsetup__camset->axis_x[0].speed);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_axisy_speed")), (gdouble)camsetup__camset->axis_y[0].speed);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_axisz1_speed")), (gdouble)camsetup__camset->axis_z[0].speed);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool1_speed")), (gdouble)camsetup__camset->axis_z[0].tool.speed);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool1_size")), (gdouble)camsetup__camset->axis_z[0].tool.head.radius);
	gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool1_type")), camsetup__camset->axis_z[0].tool.head.type);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_axisz2_speed")), (gdouble)camsetup__camset->axis_z[1].speed);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool2_speed")), (gdouble)camsetup__camset->axis_z[1].tool.speed);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool2_size")), (gdouble)camsetup__camset->axis_z[1].tool.head.radius);
	gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool2_type")), camsetup__camset->axis_z[1].tool.head.type);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool1_offset")), (gdouble)(CNC__Get_OffsetZ(gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_output"))), 0)));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool2_offset")), (gdouble)(CNC__Get_OffsetZ(gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_output"))), 1)));
	
}

//! gets the Cam-Setup-Values from in the Dialog-Gui
void camsetup__values_get (void) {
	camsetup__camset->zoom = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_zoom")));
	camsetup__camset->z_start = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_max_z"))));
	camsetup__camset->z_stop = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_min_z"))));
	camsetup__camset->z_over = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_over_z"))));
	camsetup__camset->z_steps = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_steps_z"))));
	camsetup__camset->output = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_output")));	

	camsetup__camset->axis_x[0].speed = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_axisx_speed")));
	camsetup__camset->axis_y[0].speed = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_axisy_speed")));

	camsetup__camset->axis_z[0].speed = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_axisz1_speed")));
	camsetup__camset->axis_z[0].tool.speed = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool1_speed")));
	camsetup__camset->axis_z[0].tool.head.radius = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool1_size"))));
	camsetup__camset->axis_z[0].tool.head.type = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool1_type")));	

	camsetup__camset->axis_z[1].speed = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_axisz2_speed")));
	camsetup__camset->axis_z[1].tool.speed = gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool2_speed")));
	camsetup__camset->axis_z[1].tool.head.radius = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool2_size"))));
	camsetup__camset->axis_z[1].tool.head.type = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool2_type")));	

	CNC__SetZoom(camsetup__camset->zoom);
	CNC__SetZoom_Z(1.0);
	CNC__SetOutput(camsetup__camset->output);

	gchar rcfile[1024];
	sprintf(rcfile, "%s/.camrc", g_get_home_dir());
	camsetup__save(rcfile);

}

//! GUI-Callbacks -- Cam-Setup / Help
void camsetup__help_clicked (GtkWidget *widget, void *user_data) {
}

//! GUI-Callbacks -- Cam-Setup / Apply
void camsetup__apply_clicked (GtkWidget *widget, void *user_data) {
	camsetup__values_get();
}

//! GUI-Callbacks -- Cam-Setup / Cancel
void camsetup__cancel_clicked (GtkWidget *widget, void *user_data) {
	camsetup__hide();
}

//! GUI-Callbacks -- Cam-Setup / OK
void camsetup__ok_clicked (GtkWidget *widget, void *user_data) {
	camsetup__values_get();
	camsetup__hide();
}

//! GUI-Callbacks -- Cam-Setup / Auto1
void on_camsetup_tool1_auto_clicked (GtkWidget *widget, void *user_data) {
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool1_offset")), (gdouble)(CNC__Get_OffsetZ(gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_output"))), 0)));
}

//! GUI-Callbacks -- Cam-Setup / Auto2
void on_camsetup_tool2_auto_clicked (GtkWidget *widget, void *user_data) {
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(camsetup__camset->xml, "camsetup_tool2_offset")), (gdouble)(CNC__Get_OffsetZ(gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_output"))), 1)));
}

//! GUI-Callbacks -- Cam-Setup / show CNC-Setup
void on_camsetup_general_cncsetup_clicked (GtkWidget *widget, void *user_data) {
	CNC__CfgFe_Show(gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_output"))));
}

//! GUI-Callbacks -- Cam-Setup / check CNC-Setup by Output-Selection
void on_camsetup_general_output_changed (GtkWidget *widget, void *user_data) {
	if (CNC__CfgFe_Support(gtk_combo_box_get_active(GTK_COMBO_BOX(widget))) == 1) {
		gtk_widget_set_sensitive(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_cncsetup"), TRUE);
	} else {
		gtk_widget_set_sensitive(glade_xml_get_widget(camsetup__camset->xml, "camsetup_general_cncsetup"), FALSE);
	}
}

//! runs the Cam-Setup-Dialog and wait for the replay
gint camsetup__run (gchar *title_str) {
	camsetup__values_set();
	gtk_window_set_title(GTK_WINDOW(camsetup__camset->window), title_str);
	return gtk_dialog_run(GTK_DIALOG(camsetup__camset->window));
}
