/** \file cb_app.c
 *  \brief GUI-Callbacks - Main-Functions.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <glade/glade.h>
#include <gtk/gtkgl.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "app.h"
#include "draw.h"
#include "cam_setup.h"
#include "cam.h"
#include "cam_imgload.h"
#include "cnc_output.h"
#include "cam_object.h"
#include "cb_cncctrl.h"
#include "cnc_output.h"

extern void on_glarea_realize (GtkWidget *widget, gpointer user_data);

//! Callback -- Area / Motion
void on_draw_area_motion_notify_event (GtkWidget *widget, GdkEventMotion *event) {
	gint drawable_width;
	gint drawable_height;
	drawable_width = app->draw_widget->allocation.width;
	drawable_height = app->draw_widget->allocation.height;
	gtk_ruler_set_range(GTK_RULER(glade_xml_get_widget(app->xml, "draw_xruler")),
		-3.0 * app->hm_zoom, (gdouble)((gfloat)(drawable_width + 20 - 30) / app->hm_zoom) / 10.0 * app->CamSet->zoom,
		(gdouble)((gfloat)(event->x) / app->hm_zoom) / 10.0 * app->CamSet->zoom - 3.0, (gdouble)((gfloat)(drawable_width + 20 - 30) / app->hm_zoom) / 10.0 * app->CamSet->zoom
	);
	gtk_ruler_set_range(GTK_RULER(glade_xml_get_widget(app->xml, "draw_yruler")),
		(gdouble)((gfloat)(drawable_height + 20 - 30) / app->hm_zoom) / 10.0 * app->CamSet->zoom, -3.0 * app->hm_zoom,
		(gdouble)((gfloat)(drawable_height - event->y) / app->hm_zoom) / 10.0 * app->CamSet->zoom - 3.0, (gdouble)((gfloat)(drawable_height + 20 - 30) / app->hm_zoom) / 10.0 * app->CamSet->zoom
	);
	sprintf(app->infotext, _("Possition: %0.1fmm x %0.1fmm"), (gdouble)((gfloat)(event->x) / app->hm_zoom) / 10.0 * app->CamSet->zoom, (gdouble)((gfloat)(event->y) / app->hm_zoom) / 10.0 * app->CamSet->zoom);
	app__set_infobar(app->infotext);
}

//! Callback -- Area / Reset Zoom
void on_zoom_reset_clicked (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("Zoom: 1.0"));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "gl_options_zoom_spin")), 1.0);
	app->hm_zoom = 1.0;
	app__view_redraw();
}

//! Callback -- Viewbar / Change Zoom
void on_hm_zoom_spin_changed (GtkWidget *widget, void *user_data) {
	app->hm_zoom = (gfloat)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "hm_options_zoom_spin"))));
	sprintf(app->infotext, _("HM-Zoom: %0.1f"), app->hm_zoom);
	app__set_infobar(app->infotext);
	app__view_redraw();
}

//! Callback -- Area / Redraw
void on_draw_area_expose_event (GtkWidget *widget, void *user_data) {
	app__view_redraw();
}

//! Callback -- Quit
void on_quit_activate (GtkWidget *widget, void *user_data) {
	if (CNC__OutActive == 1) {
		CNC__OutActive = 0;
	} else {
		gtk_main_quit();
	}
}

//! Callback -- Tools / mXm-Setup
void on_cnc_setup_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("CNC-Config"));
	CNC__CfgFe_Show(app->CamSet->output);
}

//! Callback -- Tools / CNC-Reset
void on_cnc_reset_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("CNC-Reset"));
	system("mxm_send -r");
}

//! Callback -- Tools / CNC-Pos 0,0
void on_cnc_pos00_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("CNC-Possition 0,0"));
	system("mxm_send -x 0");
	system("mxm_send -y 0");
}

//! Callback -- Tools / CNC-Off
void on_cnc_off_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("CNC-Off"));
	system("mxm_send -o");
}

//! Callback -- Tools / CNC-Control
void on_cnc_ctrl_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("CNC-Control"));
	cncctrl__show();
}

//! Callback -- Tools / Image-Import
void on_tools_imageimport_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("loading Image as Heightmap.."));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_use_rg")), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_invert")), FALSE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "filedialog_z_bottom")), (gdouble)app->CamSet->z_stop / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "filedialog_z_top")), (gdouble)app->CamSet->z_start / 100.0);
	app->img_filter = gtk_file_filter_new();
	gtk_file_filter_add_pixbuf_formats(app->img_filter);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(app->filedialog), app->img_filter);
	gtk_window_set_title(GTK_WINDOW(app->filedialog), _("load Image as Heightmap"));
	gtk_widget_show(app->filedialog);
	if (gtk_dialog_run(GTK_DIALOG(app->filedialog)) == GTK_RESPONSE_OK) {
		app__load_file(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(app->filedialog)), (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "filedialog_z_bottom"))) * 100.0), (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "filedialog_z_top"))) * 100.0), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_use_rg"))), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_invert"))));
	}
}

//! Callback -- Tools / 3D-Import
void on_tools_3dimport_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("loading 3D-Data as Heightmap.."));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_use_rg")), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_invert")), FALSE);
	gtk_window_set_title(GTK_WINDOW(app->filedialog), _("load 3D-Data as Heightmap"));
	app->img_filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern (app->img_filter, "*.3dm");
	gtk_file_filter_add_pattern (app->img_filter, "*.ac3");
	gtk_file_filter_add_pattern (app->img_filter, "*.cob");
	gtk_file_filter_add_pattern (app->img_filter, "*.flt");
	gtk_file_filter_add_pattern (app->img_filter, "*.heightfield");
	gtk_file_filter_add_pattern (app->img_filter, "*.iob");
	gtk_file_filter_add_pattern (app->img_filter, "*.md2");
	gtk_file_filter_add_pattern (app->img_filter, "*.md3");
	gtk_file_filter_add_pattern (app->img_filter, "*.nff");
	gtk_file_filter_add_pattern (app->img_filter, "*.obj");
	gtk_file_filter_add_pattern (app->img_filter, "*.q3o");
	gtk_file_filter_add_pattern (app->img_filter, "*.vrml");
	gtk_file_filter_add_pattern (app->img_filter, "*.lwo");
	gtk_file_filter_add_pattern (app->img_filter, "*.3ds");
	gtk_file_filter_add_pattern (app->img_filter, "*.dxf");
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(app->filedialog), app->img_filter);
	gtk_widget_show(app->filedialog);
	if (gtk_dialog_run(GTK_DIALOG(app->filedialog)) == GTK_RESPONSE_OK) {
		gchar cmd_str[1024];
		sprintf(cmd_str, "3d2hm 600 600 %s /tmp/temp.tga", gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(app->filedialog)));
		if (system(cmd_str) == 0) {
			app__load_file("/tmp/temp.tga", 0, 2000, FALSE, FALSE);
		} else {
			app__show_error("3D-Import", _("Can not convert the given file"));
		}
		system("rm -rf /tmp/temp.tga");
	}
}

//! Callback -- Tools / 2D-Import
void on_tools_2dimport_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("loading 2D-Data as Heightmap.."));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_use_rg")), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_invert")), FALSE);
	gtk_window_set_title(GTK_WINDOW(app->filedialog), _("load 2D-Data as Heightmap"));
	app->img_filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern (app->img_filter, "*.plt");
	gtk_file_filter_add_pattern (app->img_filter, "*.hpgl");
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(app->filedialog), app->img_filter);
	gtk_widget_show(app->filedialog);
	if (gtk_dialog_run(GTK_DIALOG(app->filedialog)) == GTK_RESPONSE_OK) {
		gchar cmd_str[1024];
		sprintf(cmd_str, "hp2xx -m tiff -f /tmp/temp.tiff %s", gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(app->filedialog)));
		if (system(cmd_str) == 0) {
			app__load_file("/tmp/temp.tiff", 9, 2000, FALSE, FALSE);
		} else {
			app__show_error("2D-Import", _("Can not convert the given file"));
		}
		system("rm -rf /tmp/temp.tiff");
	}
}

//! Callback -- Tools / add Frame to HeightMapSource
void on_tools_addframe_activate (GtkWidget *widget, void *user_data) {
	CAM__HeightMap *HeightMap_New;
	if ((HeightMap_New = CAM__HeightMap_AddFrame(app->HeightMapSource, 1)) != NULL) {
		CAM__Heightmap_delete(app->HeightMapSource);
		app->HeightMapSource = CAM__HeightMap_Copy(HeightMap_New);
		CAM__Heightmap_delete(HeightMap_New);
		app__showmap(app->HeightMapSource);
	}
}

//! Callback -- Tools / invert HeightMapSource
void on_tools_invert_activate (GtkWidget *widget, void *user_data) {
	CAM__HeightMap_Invert(app->HeightMapSource);
	app__showmap(app->HeightMapSource);
}

//! Callback -- Errordialog / Close
void on_errordialog_close (GtkWidget *widget, void *user_data) {
	gtk_widget_hide(glade_xml_get_widget(app->xml, "errordialog"));
}

//! Callback -- Infodialog / Close
void on_infodialog_close (GtkWidget *widget, void *user_data) {
	gtk_widget_hide(glade_xml_get_widget(app->xml, "infodialog"));
}

//! Callback -- YesNoDialog / Close
void on_yesnodialog_close (GtkWidget *widget, void *user_data) {
	gtk_widget_hide(glade_xml_get_widget(app->xml, "yesnodialog"));
}

//! Callback -- CamDialog 2D-Outliner / Close
void on_camdialog_2d_outliner_close (GtkWidget *widget, void *user_data) {
	gtk_widget_hide(glade_xml_get_widget(app->xml, "camdialog_2d_outliner"));
}

void on_view_up_clicked (GtkWidget *widget, void *user_data) {
	glRotatef(5.0, -1.0, 0.0, 0.0);
}

void on_view_down_clicked (GtkWidget *widget, void *user_data) {
	glRotatef(5.0, 1.0, 0.0, 0.0);
}

void on_view_left_clicked (GtkWidget *widget, void *user_data) {
	glRotatef(5.0, 0.0, -1.0, 0.0);
}

void on_view_right_clicked (GtkWidget *widget, void *user_data) {
	glRotatef(5.0, 0.0, 1.0, 0.0);
}

void on_view_rotate_inc_clicked (GtkWidget *widget, void *user_data) {
	glRotatef(5.0, 0.0, 0.0, -1.0);
}

void on_view_rotate_dec_clicked (GtkWidget *widget, void *user_data) {
	glRotatef(5.0, 0.0, 0.0, 1.0);
}

void on_gl_mode_clicked (GtkWidget *widget, void *user_data) {
	app->gl_mode = 1 - app->gl_mode;
	on_glarea_realize(app->gl_area, NULL);
}

void on_gl_zoom_spin_changed (GtkWidget *widget, void *user_data) {
	app->gl_zoom = (gfloat)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "gl_options_zoom_spin"))));
	sprintf(app->infotext, _("GL-Zoom: %0.1f"), app->gl_zoom);
	app__set_infobar(app->infotext);
}

void on_gl_zzoom_spin_changed (GtkWidget *widget, void *user_data) {
	app->gl_zzoom = (gfloat)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "gl_options_zzoom_spin"))));
	sprintf(app->infotext, _("GL-Zoom: %0.1f"), app->gl_zzoom);
	app__set_infobar(app->infotext);
}
