/** \file cb_cam.c
 *  \brief GUI-Callbacks - Cam-Functions
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <glade/glade.h>

#include "app.h"
#include "cam_setup.h"
#include "cam.h"
#include "cam_imgload.h"
#include "cnc_output.h"
#include "cam_object.h"

gint cam_2d_outline_do_fill = 1;
gint cam_2d_outline_do_object = 1;
gint cam_2d_outline_do_outline = 1;
gint cam_2d_outline_object_dir = 1;

//! Callback -- Cambar / CAM->CNC (2D - Object, Outline, Fill)
gpointer on_cam_2d_outline_thread (void *user_data) {
	CAM__HeightMap *HeightMap_Tmp = NULL;
	CAM__HeightMap *HeightMap_Object = NULL;
	CAM__HeightMap *HeightMap_Outline = NULL;
	gint x;
	gint y;
	gint stat;
	gint last_stat;
	gchar text_str[50];
	last_stat = 0;
	if (app != NULL && app->HeightMapSource != NULL) {
		CNC__SetOutput(app->CamSet->output);
		CNC__Start(app->HeightMapSource->width, app->HeightMapSource->height, app->CamSet->z_start, app->CamSet->axis_z[app->CamSet->tool].tool.head.radius);
		CNC__SoftStart(0);
		if (cam_2d_outline_do_outline == 1 && (HeightMap_Tmp = CAM__HeightMap_Copy(app->HeightMapSource)) != NULL) {
			if (cam_2d_outline_do_object == 1) {
				if (cam_2d_outline_object_dir == 1) {
					fprintf(stderr, "Objects: Direction=Left->Right\n");
					for (x = 0; CNC__OutActive == 1 && x < HeightMap_Tmp->width; x++) {
						stat = (gint)((100.0 / (gfloat)HeightMap_Tmp->width) * (gfloat)x);
						if (stat != last_stat) {
							sprintf(text_str, "CNC-2D / Object, >Outline<, Fill: %i%%", stat);
							app__set_infobar(text_str);
							app__set_progress(stat);
						}
						last_stat = stat;
						for (y = 0; CNC__OutActive == 1 && y < HeightMap_Tmp->height; y++) {
							if (HeightMap_Tmp->data[(y * HeightMap_Tmp->width) + x] == 65535) {
								if ((HeightMap_Object = CAM__HeightMap_MoveObject_Invert(HeightMap_Tmp, x, y)) != NULL) {
									if ((HeightMap_Outline = CAM__HeightMap_Contour(HeightMap_Object, app->CamSet)) != NULL) {
										CAM__HeightMap_CNC_Liner(HeightMap_Outline, 0, 0, HeightMap_Outline->width, HeightMap_Outline->height, app->CamSet);
										CAM__Heightmap_delete(HeightMap_Outline);
									}
									CAM__Heightmap_delete(HeightMap_Object);
								}
							}
						}
					}
				} else {
					fprintf(stderr, "Objects: Direction=Bottom->Top\n");
					for (y = 0; CNC__OutActive == 1 && y < HeightMap_Tmp->height; y++) {
						stat = (gint)((100.0 / (gfloat)HeightMap_Tmp->height) * (gfloat)y);
						if (stat != last_stat) {
							sprintf(text_str, "CNC-2D / Object, >Outline<, Fill: %i%%", stat);
							app__set_infobar(text_str);
							app__set_progress(stat);
						}
						last_stat = stat;
						for (x = 0; CNC__OutActive == 1 && x < HeightMap_Tmp->width; x++) {
							if (HeightMap_Tmp->data[(y * HeightMap_Tmp->width) + x] == 65535) {
								if ((HeightMap_Object = CAM__HeightMap_MoveObject_Invert(HeightMap_Tmp, x, y)) != NULL) {
									if ((HeightMap_Outline = CAM__HeightMap_Contour(HeightMap_Object, app->CamSet)) != NULL) {
										CAM__HeightMap_CNC_Liner(HeightMap_Outline, 0, 0, HeightMap_Outline->width, HeightMap_Outline->height, app->CamSet);
										CAM__Heightmap_delete(HeightMap_Outline);
									}
									CAM__Heightmap_delete(HeightMap_Object);
								}
							}
						}
					}
				}
				CAM__Heightmap_delete(HeightMap_Tmp);
			} else {
				app__set_infobar("Outliner...");
				if ((HeightMap_Outline = CAM__HeightMap_Contour(HeightMap_Tmp, app->CamSet)) != NULL) {
					app__set_infobar("Liner...");
					// CNC-Liner
					CAM__HeightMap_CNC_Liner(HeightMap_Outline, 0, 0, HeightMap_Outline->width, HeightMap_Outline->height, app->CamSet);
					CAM__Heightmap_delete(HeightMap_Outline);
				}
				CAM__Heightmap_delete(HeightMap_Tmp);
			}
		}
		if (cam_2d_outline_do_fill == 1 && (HeightMap_Tmp = CAM__HeightMap_Copy(app->HeightMapSource)) != NULL) {
			// Cam-Line-Filler
			app->CamSet->z_step = app->CamSet->z_start;
			app__set_infobar("Filler...");
			if ((HeightMap_Object = CAM__HeightMap_Fill(HeightMap_Tmp, app->CamSet)) != NULL) {
				app__set_infobar("Liner...");
				CAM__HeightMap_CNC_Liner(HeightMap_Object, 0, 0, HeightMap_Object->width, HeightMap_Object->height, app->CamSet);
				CAM__Heightmap_delete(HeightMap_Object);
			}
			CAM__Heightmap_delete(HeightMap_Tmp);
		}
		if (cam_2d_outline_do_fill == 2 && (HeightMap_Tmp = CAM__HeightMap_Copy(app->HeightMapSource)) != NULL) {
			// Cam-Filler
			app__set_infobar("Filler...");
			app->CamSet->z_step = app->CamSet->z_start;
			CAM__HeightMap_CNC_Fill(HeightMap_Tmp, app->CamSet);
			CAM__Heightmap_delete(HeightMap_Tmp);
		}
		app__set_infobar("CNC: Turn off");
		CNC__Off();
		CNC__Stop();
		app__set_infobar("DONE !!!");
	} else {
		app__set_infobar("No Heightmap loaded !!!");
	}
	app__set_progress(100);
	return NULL;
}

void on_cam_2d_outline_activate (GtkWidget *widget, void *user_data) {
	GThread* thread = NULL;
	if (app != NULL && app->HeightMapSource != NULL) {
		if (camsetup__run("CAM-Setup (CNC-2D / Object, Outline, Fill)") != CAMSETUP__OK) {
			return;
		}
		if (gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(app->xml, "camdialog_2d_outliner"))) != CAMSETUP__OK) {
			return;
		}
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "camdialog_2d_outliner_fill"))) == TRUE) {
			cam_2d_outline_do_fill = 1;
		} else {
			cam_2d_outline_do_fill = 0;
		}
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "camdialog_2d_outliner_objects"))) == TRUE) {
			cam_2d_outline_do_object = 1;
		} else {
			cam_2d_outline_do_object = 0;
		}
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "camdialog_2d_outliner_objects_direction"))) == TRUE) {
			cam_2d_outline_object_dir = 1;
		} else {
			cam_2d_outline_object_dir = 0;
		}
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "camdialog_2d_outliner_contour"))) == TRUE) {
			cam_2d_outline_do_outline = 1;
		} else {
			cam_2d_outline_do_outline = 0;
		}
		app__set_infobar(_("CNC-2D / Object, Outline, Fill"));
		thread = g_thread_create(on_cam_2d_outline_thread, NULL, TRUE, NULL);
/*		if(thread != NULL) {
			g_thread_join(thread);
		}
		app__view_redraw();
*/
	}
}

//! Callback -- Cambar / CAM->CNC (3D - Object, Fill & Contour)
void on_cam_3d_technical_activate (GtkWidget *widget, void *user_data) {
	CAM__HeightMap *HeightMap_Contour;
	CAM__HeightMap *HeightMap_Object;
	CAM__HeightMap *HeightMap_Depth;
	gint x;
	gint y;
	if (app != NULL && app->HeightMapSource != NULL) {
		if (camsetup__run("CAM-Setup (CNC-3D / Object,Fill,Contour)") != CAMSETUP__OK) {
			return;
		}
		app__set_infobar(_("CNC-3D / Object,Fill,Contour"));
		CNC__SetOutput(app->CamSet->output);
		CNC__Start(app->HeightMapSource->width, app->HeightMapSource->height, app->CamSet->z_start + 1, app->CamSet->axis_z[app->CamSet->tool].tool.head.radius);
		CNC__SoftStart(0);
		// for each depth
		for (app->CamSet->z_step = (app->CamSet->z_start - app->CamSet->z_steps); CNC__OutActive == 1 && app->CamSet->z_step >= app->CamSet->z_stop; app->CamSet->z_step -= app->CamSet->z_steps) {
			if ((HeightMap_Depth = CAM__HeightMap_DepthCopy(app->HeightMapSource, app->CamSet->z_step + app->CamSet->z_steps)) != NULL) {
				// for each xy-cord (search for Objects)
				for (y = 0; CNC__OutActive == 1 && y < HeightMap_Depth->height; y += 1) {
					for (x = 0; CNC__OutActive == 1 && x < HeightMap_Depth->width; x += 1) {
						if (HeightMap_Depth->data[(y * HeightMap_Depth->width) + x] <= app->CamSet->z_step) {
							// Fill Object
							if ((HeightMap_Object = CAM__HeightMap_MoveObject(HeightMap_Depth, x, y)) != NULL) {
								// Cam-Filler
								if ((HeightMap_Contour = CAM__HeightMap_Fill(HeightMap_Object, app->CamSet)) != NULL) {
									app__showmap(HeightMap_Contour);
									CAM__Heightmap_delete(HeightMap_Contour);
								}
								CAM__Heightmap_delete(HeightMap_Object);
							}
						}
					}
				}
				CAM__Heightmap_delete(HeightMap_Depth);
			}
			if ((HeightMap_Depth = CAM__HeightMap_DepthCopy(app->HeightMapSource, app->CamSet->z_step + app->CamSet->z_steps)) != NULL) {
				// for each xy-cord (search for Objects)
				for (y = 0; CNC__OutActive == 1 && y < HeightMap_Depth->height; y += 1) {
					for (x = 0; CNC__OutActive == 1 && x < HeightMap_Depth->width; x += 1) {
						if (HeightMap_Depth->data[(y * HeightMap_Depth->width) + x] <= app->CamSet->z_step) {
							// Contour Object
							if ((HeightMap_Object = CAM__HeightMap_MoveObject(HeightMap_Depth, x, y)) != NULL) {
								// Cam-Contour
								if ((HeightMap_Contour = CAM__HeightMap_Contour(HeightMap_Object, app->CamSet)) != NULL) {
									app__showmap(HeightMap_Contour);
									// CNC-Liner
									CAM__HeightMap_CNC_Liner(HeightMap_Contour, 0, 0, HeightMap_Depth->width, HeightMap_Depth->height, app->CamSet);
									CAM__Heightmap_delete(HeightMap_Contour);
								}
								CAM__Heightmap_delete(HeightMap_Object);
							}
						}
					}
				}
				CAM__Heightmap_delete(HeightMap_Depth);
			}
		}
		app->CamSet->z_step = app->CamSet->z_stop;
		CNC__Off();
		CNC__Stop();
		app__show_info("CAM->CNC (3D - Object, Fill & Contour)", _("Done !!!"));
	} else {
		app__show_error("CAM->CNC (3D - Object, Fill & Contour)", _("No Heightmap loaded !!!"));
	}
	app__set_progress(100);
}

//! Callback -- Cambar / CAM->CNC (3D - Freeform)
void on_cam_3d_freeform_activate (GtkWidget *widget, void *user_data) {
	if (app != NULL && app->HeightMapView != NULL) {
		if (camsetup__run("CAM-Setup (CNC-3D / Freeform)") != CAMSETUP__OK) {
			return;
		}
		app__set_infobar(_("CNC-3D / Freeform"));
		CNC__SetOutput(app->CamSet->output);
		CNC__Start(app->HeightMapView->width, app->HeightMapView->height, app->CamSet->z_start + 1, app->CamSet->axis_z[app->CamSet->tool].tool.head.radius);
		CNC__SoftStart(0);
		// for each depth
		for (app->CamSet->z_step = app->CamSet->z_start - app->CamSet->z_steps; CNC__OutActive == 1 && app->CamSet->z_step >= app->CamSet->z_stop; app->CamSet->z_step -= app->CamSet->z_steps) {
			while (gtk_events_pending()) {
				  gtk_main_iteration();
			}
			CAM__HeightMap_CNC_Soft3D(app->HeightMapView, app->CamSet);
		}
		if (app->CamSet->z_step > app->CamSet->z_stop) {
			app->CamSet->z_step = app->CamSet->z_stop;
			CAM__HeightMap_CNC_Soft3D(app->HeightMapView, app->CamSet);
		}
		CNC__Off();
		CNC__Stop();
		app__show_info("CAM->CNC (3D - Freeform)", _("Done !!!"));
	} else {
		app__show_error("CAM->CNC (3D - Freeform)", _("No Heightmap loaded !!!"));
	}
	app__set_progress(100);
}


//! Callback -- Cambar / Show CamSetup
void on_cam_setup_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("show CAM-Setup"));
	camsetup__show();
}
