/** \file cb_cncctrl.c
 *  \brief GUI-Callbacks - CNC-Ctrl-Functions
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <glade/glade.h>

#include "app.h"
#include "cam_setup.h"
#include "cnc_output.h"
#include "cb_cncctrl.h"

gint cncctrl_speed = 0;
gint cncctrl_x = 0;
gint cncctrl_y = 0;
gint cncctrl_z = 8000;
gint cncctrl_step_xy = 100;
gint cncctrl_step_z = 50;

void cncctrl__show (void) {
	if (camsetup__run("CAM-Setup (CNC-Control)") != CAMSETUP__OK) {
		return;
	}
	cncctrl_z = CNC__Get_OffsetZ(app->CamSet->output, -1);
	app->CamSet->zoom = 1.0;
	CNC__SetZoom(app->CamSet->zoom);
	CNC__SetOutput(app->CamSet->output);
	CNC__Start(10000, 10000, app->CamSet->z_start, 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_x_pos")), (gfloat)cncctrl_x / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_y_pos")), (gfloat)cncctrl_y / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_z_pos")), (gfloat)cncctrl_z / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_stepxy")), (gfloat)cncctrl_step_xy / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_stepz")), (gfloat)cncctrl_step_z / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_m_speed")), (gfloat)cncctrl_speed);
	gtk_widget_show(GTK_WIDGET(glade_xml_get_widget(app->xml, "cncctrl")));
}

void on_cncctrl_quit_clicked (GtkWidget *widget, void *user_data) {
	gtk_widget_hide(GTK_WIDGET(glade_xml_get_widget(app->xml, "cncctrl")));
	CNC__Off();
	CNC__Stop();
}


void on_cncctrl_m_speed_changed (GtkWidget *widget, void *user_data) {
	cncctrl_speed = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_m_speed"))));
	CNC__DrillSetSpeed(cncctrl_speed);
}
void on_cncctrl_stepxy_changed (GtkWidget *widget, void *user_data) {
	cncctrl_step_xy = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_stepxy"))) * 100.0);
}
void on_cncctrl_stepz_changed (GtkWidget *widget, void *user_data) {
	cncctrl_step_z = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_stepz"))) * 100.0);
}



void on_cncctrl_x_set_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_x = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_x_pos"))) * 100.0);
	CNC__Go_XY(cncctrl_x, cncctrl_y);
}
void on_cncctrl_x_get_clicked (GtkWidget *widget, void *user_data) {
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_x_pos")), (gfloat)cncctrl_x / 100.0);
}
void on_cncctrl_x_reset_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_x = 0;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_x_pos")), (gfloat)cncctrl_x / 100.0);
	CNC__Go_XY(cncctrl_x, cncctrl_y);
}
void on_cncctrl_x_decr_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_x -= cncctrl_step_xy;
	if (cncctrl_x < 0) {
		cncctrl_x = 0;
	}
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_x_pos")), (gfloat)cncctrl_x / 100.0);
	CNC__Go_XY(cncctrl_x, cncctrl_y);
}
void on_cncctrl_x_incr_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_x += cncctrl_step_xy;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_x_pos")), (gfloat)cncctrl_x / 100.0);
	CNC__Go_XY(cncctrl_x, cncctrl_y);
}

void on_cncctrl_y_set_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_y = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_y_pos"))) * 100.0);
	CNC__Go_XY(cncctrl_x, cncctrl_y);
}
void on_cncctrl_y_get_clicked (GtkWidget *widget, void *user_data) {
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_y_pos")), (gfloat)cncctrl_y / 100.0);
}
void on_cncctrl_y_reset_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_y = 0;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_y_pos")), (gfloat)cncctrl_y / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_y_pos")), (gfloat)cncctrl_y / 100.0);
	CNC__Go_XY(cncctrl_x, cncctrl_y);
}
void on_cncctrl_y_decr_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_y -= cncctrl_step_xy;
	if (cncctrl_y < 0) {
		cncctrl_y = 0;
	}
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_y_pos")), (gfloat)cncctrl_y / 100.0);
	CNC__Go_XY(cncctrl_x, cncctrl_y);
}
void on_cncctrl_y_incr_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_y += cncctrl_step_xy;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_y_pos")), (gfloat)cncctrl_y / 100.0);
	CNC__Go_XY(cncctrl_x, cncctrl_y);
}

void on_cncctrl_z_set_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_z = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_z_pos"))) * 100.0);
	CNC__Go_Z(cncctrl_z);
}
void on_cncctrl_z_get_clicked (GtkWidget *widget, void *user_data) {
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_z_pos")), (gfloat)cncctrl_z / 100.0);
}
void on_cncctrl_z_reset_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_z = CNC__Get_OffsetZ(app->CamSet->output, -1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_z_pos")), (gfloat)cncctrl_z / 100.0);
	CNC__Go_Z(cncctrl_z);
}
void on_cncctrl_z_decr_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_z -= cncctrl_step_z;
	if (cncctrl_z < 0) {
		cncctrl_z = 0;
	}
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_z_pos")), (gfloat)cncctrl_z / 100.0);
	CNC__Go_Z(cncctrl_z);
}
void on_cncctrl_z_incr_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_z += cncctrl_step_z;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_z_pos")), (gfloat)cncctrl_z / 100.0);
	CNC__Go_Z(cncctrl_z);
}

void on_cncctrl_toolchange_clicked (GtkWidget *widget, void *user_data) {
}
void on_cncctrl_m_decr_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_speed -= 5;
	if (cncctrl_speed < 0) {
		cncctrl_speed = 0;
	}
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_m_speed")), (gfloat)cncctrl_speed);
	CNC__DrillSetSpeed(cncctrl_speed);
}
void on_cncctrl_m_incr_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_speed += 5;
	if (cncctrl_speed > 100) {
		cncctrl_speed = 100;
	}
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_m_speed")), (gfloat)cncctrl_speed);
	CNC__DrillSetSpeed(cncctrl_speed);
}
void on_cncctrl_m_off_clicked (GtkWidget *widget, void *user_data) {
	if (cncctrl_speed == 0) {
		cncctrl_speed = 100;
	} else {
		cncctrl_speed = 0;
	}
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_m_speed")), (gfloat)cncctrl_speed);
	CNC__DrillSetSpeed(cncctrl_speed);
}

void on_cncctrl_reset_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_speed = 0;
	cncctrl_x = 0;
	cncctrl_y = 0;
	cncctrl_z = CNC__Get_OffsetZ(app->CamSet->output, -1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_x_pos")), (gfloat)cncctrl_x / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_y_pos")), (gfloat)cncctrl_y / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_z_pos")), (gfloat)cncctrl_z / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_m_speed")), (gfloat)cncctrl_speed);
	CNC__Off();
	CNC__Reset();
}
void on_cncctrl_off_clicked (GtkWidget *widget, void *user_data) {
	cncctrl_speed = 0;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "cncctrl_m_speed")), (gfloat)cncctrl_speed);
	CNC__Off();
}
void on_cncctrl_cfgfe_clicked (GtkWidget *widget, void *user_data) {
	CNC__CfgFe_Show(app->CamSet->output);
}
