/** \file cb_draw.c
 *  \brief GUI-Callbacks - Drawing-Functions.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <glade/glade.h>
#include <gtk/gtkgl.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "app.h"
#include "draw.h"


//! Callback -- Canvas / Motion
gboolean
on_canvas_layout_motion_notify_event   (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
        GtkWidget *ruler;
        gint c_offset_x, c_offset_y;
        gdouble w_offset_x, w_offset_y;
        gdouble w_width, w_height;
        gdouble w_min_x, w_min_y;
        gdouble w_pos_x, w_pos_y;
        gdouble w_max_x, w_max_y;
        gnome_canvas_get_scroll_offsets(GNOME_CANVAS(widget), &c_offset_x, &c_offset_y);
        gnome_canvas_c2w(GNOME_CANVAS(widget), c_offset_x, c_offset_y, &w_offset_x, &w_offset_y);
        gnome_canvas_c2w(GNOME_CANVAS(widget), widget->allocation.width, widget->allocation.height, &w_width, &w_height);
        gnome_canvas_c2w(GNOME_CANVAS(widget), event->x, event->y, &w_pos_x, &w_pos_y);
        gnome_canvas_get_scroll_region(GNOME_CANVAS(widget), &w_min_x, &w_min_y, &w_max_x, &w_max_y);

        ruler = glade_xml_get_widget(app->xml, "canvas_rulerx");
        gtk_ruler_set_range (GTK_RULER (ruler), w_offset_x, w_offset_x + w_width, w_pos_x, w_max_x);
        ruler = glade_xml_get_widget(app->xml, "canvas_rulery");
        gtk_ruler_set_range (GTK_RULER (ruler), w_offset_y, w_offset_y + w_height, w_pos_y, w_max_y);

        draw_motion(widget, (gint)w_pos_x, (gint)w_pos_y);
        return TRUE;
}

gboolean
on_canvas_layout_button_press_event    (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gdouble w_pos_x, w_pos_y;
	gnome_canvas_c2w(GNOME_CANVAS(widget), event->x, event->y, &w_pos_x, &w_pos_y);
	draw_start(widget, (gint)w_pos_x, (gint)w_pos_y);
	return TRUE;
}

gboolean
on_canvas_layout_button_release_event  (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gdouble w_pos_x, w_pos_y;
	gnome_canvas_c2w(GNOME_CANVAS(widget), event->x, event->y, &w_pos_x, &w_pos_y);
	draw_stop(widget, (gint)w_pos_x, (gint)w_pos_y);
	return TRUE;
}

void
on_drawbar_select_clicked          (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	draw_tool_select(0);
}

void
on_drawbar_circle_clicked          (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	draw_tool_select(1);
}

void
on_drawbar_line_clicked            (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	draw_tool_select(2);
}

void
on_drawbar_rectangle_clicked       (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	draw_tool_select(3);
}

void
on_drawbar_text_clicked            (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *widget = NULL;
	widget = glade_xml_get_widget(app->xml, "fontdialog");
	gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(widget), drawdata.fontname);
	gtk_font_selection_dialog_set_preview_text(GTK_FONT_SELECTION_DIALOG(widget), drawdata.text);
	gtk_widget_show(widget);
}


// Color-Dialog
void
on_colorbox_fill_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	draw_tooloption_fill();
}

gboolean
on_colorbox_fgpreview_button_press_event (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	drawdata.color_mode = 0;
	GtkWidget *widget2 = NULL;
	widget2 = glade_xml_get_widget(app->xml, "colordialog");
	gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(widget2)->colorsel), &drawdata.fgcolor);
	gtk_widget_show(widget2);
	return TRUE;
}

gboolean
on_colorbox_fgpreview_expose_event       (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
	GdkGC *kontext;
	kontext = gdk_gc_new(widget->window);
	gdk_gc_set_rgb_fg_color(kontext, &drawdata.fgcolor);
	gdk_draw_rectangle(widget->window, kontext, TRUE, 3, 3, widget->allocation.width - 6, widget->allocation.height - 6);
	return TRUE;
}

gboolean
on_colorbox_bgpreview_button_press_event (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	drawdata.color_mode = 1;
	GtkWidget *widget2 = NULL;
	widget2 = glade_xml_get_widget(app->xml, "colordialog");
	gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(widget2)->colorsel), &drawdata.fgcolor);
	gtk_widget_show(widget2);
	return TRUE;
}

gboolean
on_colorbox_bgpreview_expose_event       (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
	GdkGC *kontext;
	kontext = gdk_gc_new(widget->window);
	gdk_gc_set_rgb_fg_color(kontext, &drawdata.bgcolor);
	gdk_draw_rectangle(widget->window, kontext, TRUE, 3, 3, widget->allocation.width - 6, widget->allocation.height - 6);
	return TRUE;
}

void
on_colormap_depth_set                  (GtkRange        *range,
                                        gpointer         user_data)
{
	GtkAdjustment *adjustment;
	GtkWidget *preview_widget;
	adjustment = gtk_range_get_adjustment(range);
	drawdata.fgcolor.red = 65535 - (adjustment->value / 50.0) * 65535;
	drawdata.fgcolor.green = 65535 - (adjustment->value / 50.0) * 65535;
	drawdata.fgcolor.blue = 65535 - (adjustment->value / 50.0) * 65535;
	drawdata.bgcolor.red = 65535 - (adjustment->value / 50.0) * 65535;
	drawdata.bgcolor.green = 65535 - (adjustment->value / 50.0) * 65535;
	drawdata.bgcolor.blue = 65535 - (adjustment->value / 50.0) * 65535;
//	preview_widget = glade_xml_get_widget(app->xml, "colorbox_bgpreview");
//	on_colorbox_bgpreview_expose_event(preview_widget, NULL, NULL);
//	preview_widget = glade_xml_get_widget(app->xml, "colorbox_fgpreview");
//	on_colorbox_fgpreview_expose_event(preview_widget, NULL, NULL);
}

void
on_colordialog_ok_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	if (drawdata.color_mode == 1) {
		widget = glade_xml_get_widget(app->xml, "colorbox_bgpreview");
		gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(widget)->colorsel), &drawdata.bgcolor);
		on_colorbox_bgpreview_expose_event(widget, NULL, NULL);
	} else {
		widget = glade_xml_get_widget(app->xml, "colorbox_fgpreview");
		gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(widget)->colorsel), &drawdata.fgcolor);
		on_colorbox_fgpreview_expose_event(widget, NULL, NULL);
	}
	widget = glade_xml_get_widget(app->xml, "colordialog");
	gtk_widget_hide(widget);
}

void
on_colordialog_cancel_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	widget = glade_xml_get_widget(app->xml, "colordialog");
	gtk_widget_hide(widget);
}

void
on_colordialog_help_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
}


// Font-Dialog
void
on_fontdialog_apply_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	widget = glade_xml_get_widget(app->xml, "fontdialog");
	drawdata.font = gtk_font_selection_dialog_get_font(GTK_FONT_SELECTION_DIALOG(widget));
	strcpy(drawdata.fontname, gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(widget)));
	strcpy(drawdata.text, gtk_font_selection_dialog_get_preview_text(GTK_FONT_SELECTION_DIALOG(widget)));
	draw_tool_select(4);
}

void
on_fontdialog_ok_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	on_fontdialog_apply_clicked(button, user_data);
	GtkWidget *widget;
	widget = glade_xml_get_widget(app->xml, "fontdialog");
	gtk_widget_hide(widget);
}

void
on_fontdialog_cancel_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	widget = glade_xml_get_widget(app->xml, "fontdialog");
	gtk_widget_hide(widget);
}
