/** \file cb_file.c
 *  \brief GUI-Callbacks - File-Functions
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <glade/glade.h>

#include "app.h"
#include "cam_setup.h"
#include "cam.h"
#include "cam_imgload.h"
#include "cnc_output.h"
#include "cam_object.h"

//! Callback -- File Open
void on_file_open_activate (GtkWidget *widget, void *user_data) {
	app__set_infobar(_("loading Image as Heightmap.."));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_use_rg")), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_invert")), FALSE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "filedialog_z_bottom")), (gdouble)app->CamSet->z_stop / 100.0);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "filedialog_z_top")), (gdouble)app->CamSet->z_start / 100.0);
	app->img_filter = gtk_file_filter_new();
	gtk_file_filter_add_pixbuf_formats(app->img_filter);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(app->filedialog), app->img_filter);
	gtk_window_set_title(GTK_WINDOW(app->filedialog), _("load Image as Heightmap"));
	gtk_widget_show(app->filedialog);
	if (gtk_dialog_run(GTK_DIALOG(app->filedialog)) == GTK_RESPONSE_OK) {
		app__load_file(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(app->filedialog)), (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "filedialog_z_bottom"))) * 100.0), (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(app->xml, "filedialog_z_top"))) * 100.0), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_use_rg"))), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filedialog_invert"))));
	}
}

//! Callback -- File Save
void on_file_save_activate (GtkWidget *widget, void *user_data) {
	if (app != NULL && app->HeightMapView != NULL) {
		app__set_infobar(_("save Image as Heightmap.."));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filesavedialog_use_rg")), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(app->xml, "filesavedialog_invert")), FALSE);
		gtk_window_set_title(GTK_WINDOW(app->filesavedialog), _("save Image as Heightmap"));
		app->img_filter2 = gtk_file_filter_new();
		gtk_file_filter_add_pixbuf_formats(app->img_filter2);
		gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(app->filesavedialog), app->img_filter2);
		gtk_widget_show(app->filesavedialog);
		if (gtk_dialog_run(GTK_DIALOG(app->filesavedialog)) == GTK_RESPONSE_OK) {
			CAM__Heightmap_SaveTGA(app->HeightMapView, gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(app->filesavedialog)));
		}
	} else {
		app__set_infobar(_("No Heightmap loaded !!!"));
	}
}
