/** \file cnc_output.c
 *  \brief Wrapper for the CNC-Output - Plugins
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gtk/gtk.h>
#include <dirent.h>
#include <string.h>
#include <dlfcn.h>

#include "cnc_output.h"


void CNC__Halt(void) {
	CNC__OutActive = 0;
}

gint CNC__Active(void) {
	return CNC__OutActive;
}

gint CNC__Start(gint x, gint y, gint z, gint r) {
	CNC__OutActive = 1;
	return CNC__Out[CNC__OutID].cb_start(((gfloat)x * CNC__OutZoom_X), ((gfloat)y * CNC__OutZoom_Y), ((gfloat)z * CNC__OutZoom_Z), r);
}

gint CNC__Stop(void) {
	CNC__OutActive = 0;
	return CNC__Out[CNC__OutID].cb_stop();
}

gint CNC__Reset(void) {
	return CNC__Out[CNC__OutID].cb_reset();
}

gint CNC__Off(void) {
	return CNC__Out[CNC__OutID].cb_off();
}

gint CNC__Go_XY(gint x, gint y) {
	x = (gint)((gfloat)x * (gfloat)CNC__OutZoom_X);
	y = (gint)((gfloat)y * (gfloat)CNC__OutZoom_Y);
	return CNC__Out[CNC__OutID].cb_go_xy(x, y);
}

gint CNC__Go_XYZ(gint x, gint y, gint z) {
	x = (gint)((gfloat)x * (gfloat)CNC__OutZoom_X);
	y = (gint)((gfloat)y * (gfloat)CNC__OutZoom_Y);
	z = (gint)((gfloat)z * (gfloat)CNC__OutZoom_Z);
	return CNC__Out[CNC__OutID].cb_go_xyz(x, y, z);
}

gint CNC__Go_X(gint x) {
	x = (gint)((gfloat)x * (gfloat)CNC__OutZoom_X);
	return CNC__Out[CNC__OutID].cb_go_x(x);
}

gint CNC__Go_Y(gint y) {
	y = (gint)((gfloat)y * (gfloat)CNC__OutZoom_Y);
	return CNC__Out[CNC__OutID].cb_go_y(y);
}

gint CNC__Go_Z(gint z) {
//	fprintf(stderr, "Go_Z: %i\n", z);
	z = (gint)((gfloat)z * (gfloat)CNC__OutZoom_Z);
	return CNC__Out[CNC__OutID].cb_go_z(z);
}

gint CNC__DrillOn(void) {
	return CNC__Out[CNC__OutID].cb_drill_on();
}

gint CNC__DrillOff(void) {
	return CNC__Out[CNC__OutID].cb_drill_off();
}

gint CNC__DrillSetSpeed(gint speed) {
	return CNC__Out[CNC__OutID].cb_drill_set_speed(speed);
}

gint CNC__Drill(void) {
	return CNC__Out[CNC__OutID].cb_drill();
}

gint CNC__Pen(gint updown) {
	return CNC__Out[CNC__OutID].cb_pen(updown);
}

gint CNC__SpeedXY(gint speed) {
	return CNC__Out[CNC__OutID].cb_set_speed_xy(speed);
}

gint CNC__SpeedZ(gint speed) {
	return CNC__Out[CNC__OutID].cb_set_speed_z(speed);
}

gint CNC__CheckZ(void) {
	return CNC__Out[CNC__OutID].cb_check_z();
}

gint CNC__CeckSize(void) {
	return CNC__Out[CNC__OutID].cb_check_size();
}

gint CNC__ToolChange(gint tool_nr) {
	return CNC__Out[CNC__OutID].cb_tool_change(tool_nr);
}

gint CNC__SoftStart(gint value) {
	return CNC__Out[CNC__OutID].cb_soft_start(value);
}

gint CNC__LoadPlugins() {
	DIR *dir;
	struct dirent *dir_entry;
	gchar plugin_path[256];
	gchar plugin_name[20][256];
	fprintf(stderr, "Loading CNC-Output Plugins...\n");
	CNC__OutMaxID = 0;
	CNC__OutID = 0;
	if ((dir = opendir("/usr/share/cnc/output")) != NULL) {
		while ((dir_entry = readdir(dir)) != 0) {
			if (dir_entry->d_name[0] != '.') {
				strcpy(plugin_name[CNC__OutMaxID], dir_entry->d_name);
				sprintf(plugin_path, "%s/%s/%s.so", "/usr/share/cnc/output", plugin_name[CNC__OutMaxID], plugin_name[CNC__OutMaxID]);
				fprintf(stderr, "	%s (%i)...\n", plugin_name[CNC__OutMaxID], CNC__OutMaxID);
				if ((CNC__Out[CNC__OutMaxID].plugin = dlopen(plugin_path, RTLD_NOW | RTLD_GLOBAL)) != NULL) {
					CNC__Out[CNC__OutMaxID].cb_check_z = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_check_z");
					CNC__Out[CNC__OutMaxID].cb_check_size = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_check_size");
					CNC__Out[CNC__OutMaxID].cb_set_speed_xy = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_set_speed_xy");
					CNC__Out[CNC__OutMaxID].cb_set_speed_z = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_set_speed_z");
					CNC__Out[CNC__OutMaxID].cb_reset = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_reset");
					CNC__Out[CNC__OutMaxID].cb_start = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_start");
					CNC__Out[CNC__OutMaxID].cb_stop = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_stop");
					CNC__Out[CNC__OutMaxID].cb_off = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_off");
					CNC__Out[CNC__OutMaxID].cb_tool_change = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_tool_change");
					CNC__Out[CNC__OutMaxID].cb_soft_start = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_soft_start");
					CNC__Out[CNC__OutMaxID].cb_go_xy = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_go_xy");
					CNC__Out[CNC__OutMaxID].cb_go_xyz = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_go_xyz");
					CNC__Out[CNC__OutMaxID].cb_go_x = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_go_x");
					CNC__Out[CNC__OutMaxID].cb_go_y = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_go_y");
					CNC__Out[CNC__OutMaxID].cb_go_z = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_go_z");
					CNC__Out[CNC__OutMaxID].cb_drill_on = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_drill_on");
					CNC__Out[CNC__OutMaxID].cb_drill_off = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_drill_off");
					CNC__Out[CNC__OutMaxID].cb_drill_set_speed = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_drill_set_speed");
					CNC__Out[CNC__OutMaxID].cb_drill = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_drill");
					CNC__Out[CNC__OutMaxID].cb_pen = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_pen");
					CNC__Out[CNC__OutMaxID].cb_init = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_init");
					CNC__Out[CNC__OutMaxID].cb_cfgfe_show = NULL;
					CNC__Out[CNC__OutMaxID].cb_cfgfe_hide = NULL;
					CNC__Out[CNC__OutMaxID].cb_cfgfe_show = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_cfgfe_show");
					if (CNC__Out[CNC__OutMaxID].cb_cfgfe_show != NULL) {
						CNC__Out[CNC__OutMaxID].cfgfe = 1;
						CNC__Out[CNC__OutMaxID].cb_cfgfe_hide = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_cfgfe_hide");
					} else {
						CNC__Out[CNC__OutMaxID].cfgfe = 0;
					}
					CNC__Out[CNC__OutMaxID].cb_get_offset_z = NULL;
					CNC__Out[CNC__OutMaxID].cb_set_offset_z = NULL;
					CNC__Out[CNC__OutMaxID].cb_get_offset_z = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_get_offset_z");
					CNC__Out[CNC__OutMaxID].cb_set_offset_z = dlsym(CNC__Out[CNC__OutMaxID].plugin, "cncout_plugin_set_offset_z");
					sprintf(CNC__Out[CNC__OutMaxID].name, "%s", plugin_name[CNC__OutMaxID]);
					if (CNC__Out[CNC__OutMaxID].cb_init() == 0) {
						CNC__OutMaxID++;
					}
					fprintf(stderr, "	...done\n");
				} else {
					fprintf(stderr, "	...error (%s)\n", dlerror());
				}
			}
		}
		closedir(dir);
		fprintf(stderr, "...done\n");
	} else {
		fprintf(stderr, "...error (Plugin-Verzeichnis konnte nicht gefunden werden !)\n");
	}
	return 0;
}

gint CNC__SetOutput(gint output_id) {
	CNC__OutID = output_id;
	return CNC__OutID;
}

gint CNC__SetZoom_X(gfloat zoom) {
	CNC__OutZoom_X = zoom;
	return CNC__OutID;
}

gint CNC__SetZoom_Y(gfloat zoom) {
	CNC__OutZoom_Y = zoom;
	return CNC__OutID;
}

gint CNC__SetZoom_Z(gfloat zoom) {
	CNC__OutZoom_Z = zoom;
	return CNC__OutID;
}

gint CNC__SetZoom(gfloat zoom) {
	CNC__OutZoom_X = zoom;
	CNC__OutZoom_Y = zoom;
	CNC__OutZoom_Z = zoom;
	return CNC__OutID;
}

gint CNC__Init(void) {
	CNC__OutMaxID = 0;
	CNC__OutActive = 0;
	CNC__SetZoom(1.0);
	CNC__LoadPlugins();
	CNC__OutID = 3;
	return 0;
}

gint CNC__CfgFe_Show(gint plugin_id) {
	if (CNC__Out[plugin_id].cfgfe == 1) {
		return CNC__Out[plugin_id].cb_cfgfe_show();
	}
	return 1;
}

gint CNC__CfgFe_Hide(gint plugin_id) {
	if (CNC__Out[plugin_id].cfgfe == 1) {
		return CNC__Out[plugin_id].cb_cfgfe_hide();
	}
	return 1;
}

gint CNC__CfgFe_Support(gint plugin_id) {
	return CNC__Out[plugin_id].cfgfe;
}

gint CNC__Get_OffsetZ(gint plugin_id, gint tool_id) {
	if (CNC__Out[plugin_id].cb_get_offset_z != NULL) {
		return CNC__Out[plugin_id].cb_get_offset_z(tool_id);
	}
	return 0;
}

gint CNC__Set_OffsetZ(gint plugin_id, gint tool_id, gint offset) {
	if (CNC__Out[plugin_id].cb_set_offset_z != NULL) {
		return CNC__Out[plugin_id].cb_set_offset_z(tool_id, offset);
	}
	return 1;
}
