/***************************************************************************
 *            draw.c
 *
 *  Tue Mar 21 22:10:32 2006
 *  Copyright  2006  Oliver ‬Dippel
 *  o.dippel@gmx.de
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gnome.h>
#include <glade/glade.h>
#include "draw.h"
#include "app.h"


// Select-Tool
void add_select(GtkWidget *widget, gint x, gint y) {
	drawdata.active_item = gnome_canvas_get_item_at(GNOME_CANVAS(widget), x, y);
	gnome_canvas_item_set(drawdata.active_item, "fill_color", "red", NULL);
};

void move_select(GtkWidget *widget, gint x, gint y) {
};

void scale_select(GtkWidget *widget, gint x, gint y) {
};


// Rectangle-Tool
void add_rectangle(GtkWidget *widget, gint x, gint y) {
	drawdata.active_item = gnome_canvas_item_new(drawdata.active_group,
		gnome_canvas_rect_get_type(),
		"x1", (double) (x),
		"y1", (double) (y),
		"x2", (double) (x + 1),
		"y2", (double) (y + 1),
		"outline_color_gdk", &drawdata.fgcolor,
		"width_units", 1.0,
	NULL);
	if (drawdata.fill == 1) {
		gnome_canvas_item_set(drawdata.active_item,	"fill-color-gdk", &drawdata.bgcolor, NULL);
	}
	gnome_canvas_item_raise_to_top(drawdata.active_item);
	gnome_canvas_item_show(drawdata.active_item);
};

void move_rectangle(GtkWidget *widget, gint x, gint y) {
	gnome_canvas_item_set(drawdata.active_item,
		"x2", (double) (x),
		"y2", (double) (y),
	NULL);
};

void scale_rectangle(GtkWidget *widget, gint x, gint y) {
	gnome_canvas_item_set(drawdata.active_item,
		"x2", (double) (x),
		"y2", (double) (y),
	NULL);
};


// Circle-Tool
void add_circle(GtkWidget *widget, gint x, gint y) {
	drawdata.active_item = gnome_canvas_item_new(drawdata.active_group,
		gnome_canvas_ellipse_get_type(),
		"x1", (double) (x),
		"y1", (double) (y),
		"x2", (double) (x + 1),
		"y2", (double) (y + 1),
		"outline_color_gdk", &drawdata.fgcolor,
		"width_units", 1.0,
	NULL);
	if (drawdata.fill == 1) {
		gnome_canvas_item_set(drawdata.active_item,	"fill-color-gdk", &drawdata.bgcolor, NULL);
	}
	gnome_canvas_item_raise_to_top(drawdata.active_item);
	gnome_canvas_item_show(drawdata.active_item);
};

void move_circle(GtkWidget *widget, gint x, gint y) {
};

void scale_circle(GtkWidget *widget, gint x, gint y) {
	gnome_canvas_item_set(drawdata.active_item,
		"x2", (double) (x),
		"y2", (double) (y),
	NULL);
};


// Line-Tool
void add_line(GtkWidget *widget, gint x, gint y) {
	drawdata.canvas_points->coords[0] = x;
	drawdata.canvas_points->coords[1] = y;
	drawdata.canvas_points->coords[2] = x;
	drawdata.canvas_points->coords[3] = y;
	drawdata.active_item = gnome_canvas_item_new(drawdata.active_group,
		gnome_canvas_line_get_type(),	
		"points", drawdata.canvas_points,
		"outline_color_gdk", &drawdata.fgcolor,
		"width_units", (double) 1,
		/*
		"last_arrowhead", TRUE,
		"arrow_shape_a", 9.0,
		"arrow_shape_b", 13.0,
		"arrow_shape_c", 2.0,
		*/
	NULL);
};

void move_line(GtkWidget *widget, gint x, gint y) {
};

void scale_line(GtkWidget *widget, gint x, gint y) {
	drawdata.canvas_points->coords[2] = x;
	drawdata.canvas_points->coords[3] = y;
	gnome_canvas_item_set(drawdata.active_item,
		"points", drawdata.canvas_points,
	NULL);
};


// Text-Tool
void add_text(GtkWidget *widget, gint x, gint y) {
	drawdata.canvas_points->coords[1] = y;
	drawdata.active_item = gnome_canvas_item_new(drawdata.active_group,
		gnome_canvas_text_get_type (),
		"x", (double) x,
		"y", (double) y,
		"font", drawdata.fontname,
		"size_points", (double) 2,
		"anchor", GTK_ANCHOR_NORTH_WEST,
		"fill_color_gdk", &drawdata.bgcolor,
		"text", drawdata.text,
	NULL);
};

void move_text(GtkWidget *widget, gint x, gint y) {
};

void scale_text(GtkWidget *widget, gint x, gint y) {
	gnome_canvas_item_set(drawdata.active_item,
		"size_points", (double)(y - drawdata.canvas_points->coords[1]),
	NULL);
};

// Wrapper
void draw_start(GtkWidget *widget, gint x, gint y) {
	drawdata.mode = 1;
	drawdata.tool_add_cb[drawdata.tool](widget, x, y);
};

void draw_status(GtkWidget *widget, gint x, gint y) {
	GtkWidget *statusbar;
	gchar infotext[1024];
	statusbar = glade_xml_get_widget(app->xml, "statusbar");

	sprintf(infotext, "Mouse-Position: %0.2f mm , %0.2f mm", (float)x / 100.0, (float)y / 100.0);
	gtk_statusbar_push(GTK_STATUSBAR(statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "Mouse-Position"), infotext);
};

void draw_motion(GtkWidget *widget, gint x, gint y) {
	if (drawdata.mode == 1) {
		drawdata.tool_scale_cb[drawdata.tool](widget, x, y);
	}
	draw_status(widget, x, y);
};

void draw_stop(GtkWidget *widget, gint x, gint y) {
	if (drawdata.mode == 1) {
		drawdata.mode = 0;
		drawdata.tool_scale_cb[drawdata.tool](widget, x, y);
	}
};


// Misc
void draw_tool_select(gint tool) {
	drawdata.tool = tool;
};

void draw_tooloption_fill(void) {
	drawdata.fill = 1 - drawdata.fill;
};

void draw_init(void) {
	drawdata.tool = 3;
	drawdata.mode = 0;
	drawdata.fill = 0;
	drawdata.color_mode = 0;
	drawdata.fgcolor.red = 0;
	drawdata.fgcolor.green = 0;
	drawdata.fgcolor.blue = 0;
	drawdata.bgcolor.red = 65535;
	drawdata.bgcolor.green = 65535;
	drawdata.bgcolor.blue = 65535;
	drawdata.active_item = NULL;
	drawdata.active_group = NULL;
	drawdata.tool_add_cb[0] = add_select;
	drawdata.tool_add_cb[1] = add_circle;
	drawdata.tool_add_cb[2] = add_line;
	drawdata.tool_add_cb[3] = add_rectangle;
	drawdata.tool_add_cb[4] = add_text;
	drawdata.tool_move_cb[0] = move_select;
	drawdata.tool_move_cb[1] = move_circle;
	drawdata.tool_move_cb[2] = move_line;
	drawdata.tool_move_cb[3] = move_rectangle;
	drawdata.tool_move_cb[4] = move_text;
	drawdata.tool_scale_cb[0] = scale_select;
	drawdata.tool_scale_cb[1] = scale_circle;
	drawdata.tool_scale_cb[2] = scale_line;
	drawdata.tool_scale_cb[3] = scale_rectangle;
	drawdata.tool_scale_cb[4] = scale_text;
	GtkWidget *widget;
	widget = glade_xml_get_widget(app->xml, "canvas_layout");
	gnome_canvas_set_scroll_region(GNOME_CANVAS(widget), 0, 0, (int)(1800), (int)(1600));
	drawdata.canvas_points = gnome_canvas_points_new (2);
	strcpy(drawdata.fontname, "verdana bold");
	strcpy(drawdata.text, "Text...");
	drawdata.active_group = gnome_canvas_root(GNOME_CANVAS(widget));
};
