/** \file gl.c
 *  \brief OpenGL-Preview.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <gtk/gtkgl.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <glade/glade.h>

#include "app.h"
#include "g3d.h"

gint gl__export_heightmap_tga(const char *filename, int width, int height) {
	FILE *f = NULL;
	int x, y, z;
	float z_data[1];
	if ((f = fopen( filename, "w" )) != NULL) {
		fputc (0x00, f);	/* ID Length, 0 => No ID	*/
		fputc (0x00, f);	/* Color Map Type, 0 => No color map included	*/
		fputc (0x02, f);	/* Image Type, 2 => Uncompressed, True-color Image */
		fputc (0x00, f);	/* Next five bytes are about the color map entries */
		fputc (0x00, f);	/* 2 bytes Index, 2 bytes length, 1 byte size */
		fputc (0x00, f);
		fputc (0x00, f);
		fputc (0x00, f);
		fputc (0x00, f);	/* X-origin of Image	*/
		fputc (0x00, f);
		fputc (0x00, f);	/* Y-origin of Image	*/
		fputc (0x00, f);
		fputc (width & 0xff, f);      /* Image Width	*/
		fputc ((width>>8) & 0xff, f);
		fputc (height & 0xff, f);     /* Image Height	*/
		fputc ((height>>8) & 0xff, f);
		fputc (0x18, f);		/* Pixel Depth, 0x18 => 24 Bits	*/
		fputc (0x20, f);		/* Image Descriptor	*/
		fclose(f);
		f = fopen( filename, "ab" );  /* reopen in binary append mode */
		for (y = height - 1; y >= 0; y--) {
			//fprintf(stderr, "%i\n", 100 - (100 / height * y));
			for (x = 0; x < width; x++) {
				glReadPixels(x, y, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, z_data);
				z = (int)(z_data[0] * 255);
				fputc(z, f);
				fputc(z, f);
				fputc(z, f);
			}
		}
		fclose(f);
		return 0;
	}
	return 1;
}

void gl__draw (void) {
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(app->gl_zoom, app->gl_zoom, app->gl_zoom);
	glCallList(1);
	glPopMatrix();
	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(app->gl_area);
	if (gdk_gl_drawable_is_double_buffered(gldrawable)) {
		gdk_gl_drawable_swap_buffers(gldrawable);
	} else {
		glFlush();
	}
/*
	static gint start_flag = 10;
	if (start_flag == 0) {
		start_flag = -1;
		fprintf(stderr, "%i x %i\n", app->gl_area->allocation.width, app->gl_area->allocation.height);
		gl__export_heightmap_tga("/tmp/out.tga", app->gl_area->allocation.width, app->gl_area->allocation.height);
	} else if (start_flag > 0) {
		start_flag--;
	}
*/
}

void gl__draw_heightmap (void) {
	GdkPixmap *pixmap = NULL;
	GdkGLDrawable *gldrawable;
	if (pixmap != NULL) {
		g_object_unref(G_OBJECT(pixmap));
	}
	pixmap = gdk_pixmap_new(app->gl_area->window, 1000, 1000, -1);
  	gldrawable = GDK_GL_DRAWABLE(gdk_pixmap_set_gl_capability(pixmap, app->gl_config, NULL));
	GdkGLContext *glcontext = NULL;
	if (glcontext == NULL) {
		glcontext = gdk_gl_context_new (gldrawable, NULL, FALSE, GDK_GL_RGBA_TYPE);
		if (glcontext == NULL) {
			g_print ("Connot create the OpenGL rendering context\n");
			if (gtk_main_level () != 0) {
				gtk_main_quit ();
			}
			return;
		}
		g_print ("The OpenGL rendering context is created\n");
	}
	if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext)) {
		return;
	}
	glViewport (0, 0, 640, 480);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glCallList (1);
	glFlush ();
	gdk_gl_drawable_gl_end (gldrawable);
}

void
on_glarea_realize                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GdkGLContext *glcontext = NULL;
	GdkGLDrawable *gldrawable = NULL;
	glcontext = gtk_widget_get_gl_context (app->gl_area);
	gldrawable = gtk_widget_get_gl_drawable (app->gl_area);
	if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext)) {
		return;
	}
	if (app != NULL && app->HeightMapView != NULL) {
		glNewList(1, GL_COMPILE);
		CAM__HeightMap_DrawGL(app->HeightMapView, app->gl_mode);
		glEndList();
//	} else {
//		glNewList(1, GL_COMPILE);
//		g3d_load("/tmp/taximax.3ds");
//		g3d_view();
//		g3d_close();
//		glEndList();
	}

	gdk_gl_drawable_gl_end(gldrawable);
}


gboolean
on_glarea_configure_event             (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{
	GdkGLContext *glcontext = gtk_widget_get_gl_context(widget);
	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);
	if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext)) {
		return FALSE;
	}
	glViewport(0, 0, widget->allocation.width, widget->allocation.height);
	gdk_gl_drawable_gl_end(gldrawable);
	return TRUE;
}

gboolean
on_glarea_expose_event                (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
	GdkGLContext *glcontext = gtk_widget_get_gl_context (app->gl_area);
	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (app->gl_area);
	if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext)) {
		return FALSE;
	}
	gl__draw();
	gdk_gl_drawable_gl_end (gldrawable);
	return FALSE;
}

void gl__window_new (void) {
	// OpenGL-Init
	app->gl_config = gdk_gl_config_new_by_mode(GDK_GL_MODE_RGB | GDK_GL_MODE_DEPTH | GDK_GL_MODE_DOUBLE);
	if (app->gl_config == NULL) {
		g_print ("*** Cannot find the double-buffered visual.\n");
		g_print ("*** Trying single-buffered visual.\n");
		app->gl_config = gdk_gl_config_new_by_mode(GDK_GL_MODE_RGB | GDK_GL_MODE_DEPTH);
		if (app->gl_config == NULL) {
			g_print ("*** No appropriate OpenGL-capable visual found.\n");
			exit(1);
		}
    }
	// Create the OpenGL-Widget
	app->gl_area = gtk_drawing_area_new();
	gtk_widget_show(app->gl_area);
	gtk_widget_set_events(app->gl_area, GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_MOTION_MASK | GDK_BUTTON1_MOTION_MASK | GDK_BUTTON2_MOTION_MASK | GDK_BUTTON3_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
	g_signal_connect((gpointer) app->gl_area, "realize", G_CALLBACK (on_glarea_realize), NULL);
	g_signal_connect((gpointer) app->gl_area, "configure_event", G_CALLBACK (on_glarea_configure_event), NULL);
	g_signal_connect_after((gpointer) app->gl_area, "expose_event", G_CALLBACK (on_glarea_expose_event), NULL);
	gtk_widget_set_gl_capability(app->gl_area, app->gl_config, NULL, TRUE, GDK_GL_RGBA_TYPE);
	gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(app->xml, "gl_box")), app->gl_area);
	// Setup-Lights
	static GLfloat light0_pos[4] = {-50.0, 50.0, 0.0, 0.0};
	static GLfloat light0_color[4] = {.6, .6, .6, 1.0};
	static GLfloat light1_pos[4] = { 50.0, 50.0, 0.0, 0.0};
	static GLfloat light1_color[4] = {0.3, 0.3, 0.4, 1.0};
	glLightfv(GL_LIGHT0, GL_POSITION, light0_pos);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_color);
	glLightfv(GL_LIGHT1, GL_POSITION, light1_pos);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_color);
//	glEnable(GL_LIGHT0);
//	glEnable(GL_LIGHT1);
//	glEnable(GL_LIGHTING);
//	glEnable(GL_COLOR_MATERIAL);
	// Setup-View
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_DITHER);
	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
	glViewport(0, 0, app->gl_area->allocation.width, app->gl_area->allocation.height);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
//	glOrtho(-12.5, 12.5, -12.5, 12.5, -12.0, 12.0);
	gluPerspective(100.0, 1.0, 1.0, 100.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.0, 0.0, 3.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
	glTranslatef(0.0, 0.0, -13.0);
	glClearColor(0.6, 0.7, 1.0, 1.0);
	glClearDepth(1.0);
}
